@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -42,7 +42,6 @@ typedef struct {
 typedef struct gnutls_certificate_credentials_st
 {
   gnutls_dh_params_t dh_params;
-  gnutls_rsa_params_t rsa_params;
   /* this callback is used to retrieve the DH or RSA
    * parameters.
    */
@@ -87,6 +86,8 @@ typedef struct gnutls_certificate_credentials_st
   gnutls_certificate_verify_function *verify_callback;
 
   struct pin_info_st pin;
+  /* temporarily hold the PIN if set_key_file2() is used with a PIN */
+  char pin_tmp[GNUTLS_PKCS11_MAX_PIN_LEN];
   
   /* OCSP */
   gnutls_status_request_ocsp_func ocsp_func;
@@ -109,7 +110,6 @@ typedef struct cert_auth_info_st
    * They must be freed.
    */
   dh_info_st dh;
-  rsa_info_st rsa_export;
 
   gnutls_datum_t *raw_certificate_list; /* holds the raw certificate of the
                                          * peer.
@@ -145,13 +145,7 @@ void _gnutls_selected_certs_deinit (gnutls_session_t session);
 void _gnutls_selected_certs_set (gnutls_session_t session,
                                  gnutls_pcert_st * certs, int ncerts,
                                  gnutls_privkey_t key, int need_free);
-
-gnutls_rsa_params_t _gnutls_certificate_get_rsa_params (gnutls_rsa_params_t
-                                                        rsa_params,
-                                                        gnutls_params_function
-                                                        * func,
-                                                        gnutls_session_t);
-
+ 
 int _gnutls_get_auth_info_pcert (gnutls_pcert_st* gcert,
                                  gnutls_certificate_type_t type,
                                  cert_auth_info_t info);
@@ -169,4 +163,10 @@ int _gnutls_selected_cert_supported_kx (struct gnutls_session_int *session,
 int
 _gnutls_check_key_cert_match (gnutls_certificate_credentials_t res);
 
+int _gnutls_gen_dhe_signature(gnutls_session_t session, gnutls_buffer_st* data,
+	uint8_t* plain, unsigned plain_size);
+int
+_gnutls_proc_dhe_signature (gnutls_session_t session, uint8_t * data,
+                    size_t _data_size, gnutls_datum_t *vparams);
+
 #endif
