@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -26,6 +26,7 @@
 #ifdef STDC_HEADERS
 #include <stdarg.h>
 #endif
+#include "vasprintf.h"
 
 /* I18n of error codes. */
 #include "gettext.h"
@@ -93,7 +94,10 @@ static const gnutls_error_entry error_algorithms[] = {
                GNUTLS_E_NO_CERTIFICATE_FOUND, 1),
   ERROR_ENTRY (N_("The given DSA key is incompatible with the selected TLS protocol."),
                GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL, 1),
-
+  ERROR_ENTRY (N_("A heartbeat pong message was received."),
+               GNUTLS_E_HEARTBEAT_PONG_RECEIVED, 0),
+  ERROR_ENTRY (N_("A heartbeat ping message was received."),
+               GNUTLS_E_HEARTBEAT_PING_RECEIVED, 0),
   ERROR_ENTRY (N_("There is already a crypto algorithm with lower priority."),
                GNUTLS_E_CRYPTO_ALREADY_REGISTERED, 1),
 
@@ -151,6 +155,8 @@ static const gnutls_error_entry error_algorithms[] = {
                GNUTLS_E_RECORD_LIMIT_REACHED, 1),
   ERROR_ENTRY (N_("Error in the certificate."), GNUTLS_E_CERTIFICATE_ERROR,
                1),
+  ERROR_ENTRY (N_("Could not authenticate peer."), GNUTLS_E_AUTH_ERROR,
+               1),
   ERROR_ENTRY (N_("Unknown Subject Alternative name in X.509 certificate."),
                GNUTLS_E_X509_UNKNOWN_SAN, 1),
 
@@ -344,6 +350,14 @@ static const gnutls_error_entry error_algorithms[] = {
                GNUTLS_E_PKCS11_REQUESTED_OBJECT_NOT_AVAILBLE, 1),
   ERROR_ENTRY (N_("The provided X.509 certificate list is not sorted (in subject to issuer order)"),
                GNUTLS_E_CERTIFICATE_LIST_UNSORTED, 1),
+  ERROR_ENTRY (N_("The OCSP response is invalid"),
+               GNUTLS_E_OCSP_RESPONSE_ERROR, 1),
+  ERROR_ENTRY (N_("There is no certificate status (OCSP)."),
+               GNUTLS_E_NO_CERTIFICATE_STATUS, 1),
+  ERROR_ENTRY (N_("Error in the system's randomness device."),
+               GNUTLS_E_RANDOM_DEVICE_ERROR, 1),
+  ERROR_ENTRY (N_("No common application protocol could be negotiated."),
+               GNUTLS_E_NO_APPLICATION_PROTOCOL, 1),
   {NULL, NULL, 0, 0}
 };
 
