@@ -27,64 +27,35 @@
 #include <gnutls_int.h>
 #include <gnutls_hash_int.h>
 #include <gnutls_errors.h>
-
-static size_t
-digest_length (int algo)
-{
-  switch (algo)
-    {
-    case GNUTLS_DIG_NULL:
-    case GNUTLS_MAC_AEAD:
-      return 0;
-    case GNUTLS_DIG_MD5:
-    case GNUTLS_DIG_MD2:
-      return 16;
-    case GNUTLS_DIG_SHA1:
-    case GNUTLS_DIG_RMD160:
-      return 20;
-    case GNUTLS_DIG_SHA256:
-      return 32;
-    case GNUTLS_DIG_SHA384:
-      return 48;
-    case GNUTLS_DIG_SHA512:
-      return 64;
-    case GNUTLS_DIG_SHA224:
-      return 28;
-    default:
-      gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
-    }
-}
-
+#include <algorithms.h>
 
 int
-_gnutls_hash_init (digest_hd_st * dig, gnutls_digest_algorithm_t algorithm)
+_gnutls_hash_init (digest_hd_st * dig, const mac_entry_st* e)
 {
   int result;
   const gnutls_crypto_digest_st *cc = NULL;
 
-  dig->algorithm = algorithm;
+  dig->e = e;
 
   /* check if a digest has been registered 
    */
-  cc = _gnutls_get_crypto_digest (algorithm);
+  cc = _gnutls_get_crypto_digest (e->id);
   if (cc != NULL && cc->init)
     {
-      if (cc->init (algorithm, &dig->handle) < 0)
+      if (cc->init (e->id, &dig->handle) < 0)
         {
           gnutls_assert ();
           return GNUTLS_E_HASH_FAILED;
         }
 
       dig->hash = cc->hash;
-      dig->reset = cc->reset;
       dig->output = cc->output;
       dig->deinit = cc->deinit;
 
       return 0;
     }
 
-  result = _gnutls_digest_ops.init (algorithm, &dig->handle);
+  result = _gnutls_digest_ops.init (e->id, &dig->handle);
   if (result < 0)
     {
       gnutls_assert ();
@@ -92,7 +63,6 @@ _gnutls_hash_init (digest_hd_st * dig, gnutls_digest_algorithm_t algorithm)
     }
 
   dig->hash = _gnutls_digest_ops.hash;
-  dig->reset = _gnutls_digest_ops.reset;
   dig->output = _gnutls_digest_ops.output;
   dig->deinit = _gnutls_digest_ops.deinit;
 
@@ -114,14 +84,6 @@ _gnutls_hash_deinit (digest_hd_st * handle, void *digest)
   handle->handle = NULL;
 }
 
-/* returns the output size of the given hash/mac algorithm
- */
-size_t
-_gnutls_hash_get_algo_len (gnutls_digest_algorithm_t algorithm)
-{
-  return digest_length (algorithm);
-}
-
 int
 _gnutls_hash_fast (gnutls_digest_algorithm_t algorithm,
                    const void *text, size_t textlen, void *digest)
@@ -157,7 +119,7 @@ _gnutls_hash_fast (gnutls_digest_algorithm_t algorithm,
 /* HMAC interface */
 
 int
-_gnutls_hmac_fast (gnutls_mac_algorithm_t algorithm, const void *key,
+_gnutls_mac_fast (gnutls_mac_algorithm_t algorithm, const void *key,
                    int keylen, const void *text, size_t textlen, void *digest)
 {
   int ret;
@@ -168,7 +130,7 @@ _gnutls_hmac_fast (gnutls_mac_algorithm_t algorithm, const void *key,
   cc = _gnutls_get_crypto_mac (algorithm);
   if (cc != NULL)
     {
-      if (cc->fast (algorithm, key, keylen, text, textlen, digest) < 0)
+      if (cc->fast (algorithm, NULL, 0, key, keylen, text, textlen, digest) < 0)
         {
           gnutls_assert ();
           return GNUTLS_E_HASH_FAILED;
@@ -177,7 +139,7 @@ _gnutls_hmac_fast (gnutls_mac_algorithm_t algorithm, const void *key,
       return 0;
     }
 
-  ret = _gnutls_mac_ops.fast (algorithm, key, keylen, text, textlen, digest);
+  ret = _gnutls_mac_ops.fast (algorithm, NULL, 0, key, keylen, text, textlen, digest);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -191,7 +153,7 @@ _gnutls_hmac_fast (gnutls_mac_algorithm_t algorithm, const void *key,
 /* Returns true(non-zero) or false(0) if the 
  * provided hash exists
  */
-int _gnutls_hmac_exists(gnutls_mac_algorithm_t algo)
+int _gnutls_mac_exists(gnutls_mac_algorithm_t algo)
 {
   const gnutls_crypto_mac_st *cc = NULL;
 
@@ -202,58 +164,57 @@ int _gnutls_hmac_exists(gnutls_mac_algorithm_t algo)
 }
 
 int
-_gnutls_hmac_init (digest_hd_st * dig, gnutls_mac_algorithm_t algorithm,
+_gnutls_mac_init (mac_hd_st * mac, const mac_entry_st* e,
                    const void *key, int keylen)
 {
   int result;
   const gnutls_crypto_mac_st *cc = NULL;
 
-  dig->algorithm = (gnutls_digest_algorithm_t)algorithm;
-  dig->key = key;
-  dig->keysize = keylen;
+  mac->e = e;
+  mac->mac_len = _gnutls_mac_get_algo_len (e);
 
   /* check if a digest has been registered 
    */
-  cc = _gnutls_get_crypto_mac (algorithm);
+  cc = _gnutls_get_crypto_mac (e->id);
   if (cc != NULL && cc->init != NULL)
     {
-      if (cc->init (algorithm, &dig->handle) < 0)
+      if (cc->init (e->id, &mac->handle) < 0)
         {
           gnutls_assert ();
           return GNUTLS_E_HASH_FAILED;
         }
 
-      if (cc->setkey (dig->handle, key, keylen) < 0)
+      if (cc->setkey (mac->handle, key, keylen) < 0)
         {
           gnutls_assert ();
-          cc->deinit (dig->handle);
+          cc->deinit (mac->handle);
           return GNUTLS_E_HASH_FAILED;
         }
 
-      dig->hash = cc->hash;
-      dig->output = cc->output;
-      dig->deinit = cc->deinit;
-      dig->reset = cc->reset;
+      mac->hash = cc->hash;
+      mac->setnonce = cc->setnonce;
+      mac->output = cc->output;
+      mac->deinit = cc->deinit;
 
       return 0;
     }
 
-  result = _gnutls_mac_ops.init (algorithm, &dig->handle);
+  result = _gnutls_mac_ops.init (e->id, &mac->handle);
   if (result < 0)
     {
       gnutls_assert ();
       return result;
     }
 
-  dig->hash = _gnutls_mac_ops.hash;
-  dig->output = _gnutls_mac_ops.output;
-  dig->deinit = _gnutls_mac_ops.deinit;
-  dig->reset = _gnutls_mac_ops.reset;
+  mac->hash = _gnutls_mac_ops.hash;
+  mac->setnonce = _gnutls_mac_ops.setnonce;
+  mac->output = _gnutls_mac_ops.output;
+  mac->deinit = _gnutls_mac_ops.deinit;
 
-  if (_gnutls_mac_ops.setkey (dig->handle, key, keylen) < 0)
+  if (_gnutls_mac_ops.setkey (mac->handle, key, keylen) < 0)
     {
       gnutls_assert();
-      dig->deinit(dig->handle);
+      mac->deinit(mac->handle);
       return GNUTLS_E_HASH_FAILED;
     }
 
@@ -261,7 +222,7 @@ _gnutls_hmac_init (digest_hd_st * dig, gnutls_mac_algorithm_t algorithm,
 }
 
 void
-_gnutls_hmac_deinit (digest_hd_st * handle, void *digest)
+_gnutls_mac_deinit (mac_hd_st * handle, void *digest)
 {
   if (handle->handle == NULL)
     {
@@ -269,7 +230,7 @@ _gnutls_hmac_deinit (digest_hd_st * handle, void *digest)
     }
 
   if (digest)
-    _gnutls_hmac_output (handle, digest);
+    _gnutls_mac_output (handle, digest);
 
   handle->deinit (handle->handle);
   handle->handle = NULL;
@@ -293,13 +254,13 @@ get_padsize (gnutls_digest_algorithm_t algorithm)
  */
 
 int
-_gnutls_mac_init_ssl3 (digest_hd_st * ret, gnutls_mac_algorithm_t algorithm,
+_gnutls_mac_init_ssl3 (digest_hd_st * ret, const mac_entry_st* e,
                        void *key, int keylen)
 {
   uint8_t ipad[48];
   int padsize, result;
 
-  padsize = get_padsize ((gnutls_digest_algorithm_t)algorithm);
+  padsize = get_padsize ((gnutls_digest_algorithm_t)e->id);
   if (padsize == 0)
     {
       gnutls_assert ();
@@ -308,7 +269,7 @@ _gnutls_mac_init_ssl3 (digest_hd_st * ret, gnutls_mac_algorithm_t algorithm,
 
   memset (ipad, 0x36, padsize);
 
-  result = _gnutls_hash_init (ret, (gnutls_digest_algorithm_t)algorithm);
+  result = _gnutls_hash_init (ret, e);
   if (result < 0)
     {
       gnutls_assert ();
@@ -325,25 +286,6 @@ _gnutls_mac_init_ssl3 (digest_hd_st * ret, gnutls_mac_algorithm_t algorithm,
   return 0;
 }
 
-void
-_gnutls_mac_reset_ssl3 (digest_hd_st * handle)
-{
-  uint8_t ipad[48];
-  int padsize;
-
-  padsize = get_padsize (handle->algorithm);
-
-  memset (ipad, 0x36, padsize);
-
-  _gnutls_hash_reset(handle);
-
-  if (handle->keysize > 0)
-    _gnutls_hash (handle, handle->key, handle->keysize);
-  _gnutls_hash (handle, ipad, padsize);
-
-  return;
-}
-
 int 
 _gnutls_mac_output_ssl3 (digest_hd_st * handle, void *digest)
 {
@@ -353,7 +295,7 @@ _gnutls_mac_output_ssl3 (digest_hd_st * handle, void *digest)
   int padsize;
   int block, rc;
 
-  padsize = get_padsize (handle->algorithm);
+  padsize = get_padsize (handle->e->id);
   if (padsize == 0)
     {
       gnutls_assert ();
@@ -362,7 +304,7 @@ _gnutls_mac_output_ssl3 (digest_hd_st * handle, void *digest)
 
   memset (opad, 0x5C, padsize);
 
-  rc = _gnutls_hash_init (&td, handle->algorithm);
+  rc = _gnutls_hash_init (&td, handle->e);
   if (rc < 0)
     {
       gnutls_assert ();
@@ -373,12 +315,19 @@ _gnutls_mac_output_ssl3 (digest_hd_st * handle, void *digest)
     _gnutls_hash (&td, handle->key, handle->keysize);
 
   _gnutls_hash (&td, opad, padsize);
-  block = _gnutls_hmac_get_algo_len (handle->algorithm);
+  block = _gnutls_mac_get_algo_len (handle->e);
   _gnutls_hash_output (handle, ret);    /* get the previous hash */
   _gnutls_hash (&td, ret, block);
 
   _gnutls_hash_deinit (&td, digest);
   
+  /* reset handle */
+  memset (opad, 0x36, padsize);
+
+  if (handle->keysize > 0)
+    _gnutls_hash (handle, handle->key, handle->keysize);
+  _gnutls_hash (handle, opad, padsize);
+  
   return 0;
 }
 
@@ -405,7 +354,7 @@ _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
   int padsize;
   int block, rc;
 
-  padsize = get_padsize (handle->algorithm);
+  padsize = get_padsize (handle->e->id);
   if (padsize == 0)
     {
       gnutls_assert ();
@@ -416,7 +365,7 @@ _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
   memset (opad, 0x5C, padsize);
   memset (ipad, 0x36, padsize);
 
-  rc = _gnutls_hash_init (&td, handle->algorithm);
+  rc = _gnutls_hash_init (&td, handle->e);
   if (rc < 0)
     {
       gnutls_assert ();
@@ -427,7 +376,7 @@ _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
     _gnutls_hash (&td, key, key_size);
 
   _gnutls_hash (&td, opad, padsize);
-  block = _gnutls_hmac_get_algo_len (handle->algorithm);
+  block = _gnutls_mac_get_algo_len (handle->e);
 
   if (key_size > 0)
     _gnutls_hash (handle, key, key_size);
@@ -459,7 +408,7 @@ ssl3_sha (int i, uint8_t * secret, int secret_len,
       text1[j] = 65 + i;        /* A==65 */
     }
 
-  ret = _gnutls_hash_init (&td, GNUTLS_MAC_SHA1);
+  ret = _gnutls_hash_init (&td, mac_to_entry(GNUTLS_MAC_SHA1));
   if (ret < 0)
     {
       gnutls_assert ();
@@ -485,7 +434,7 @@ ssl3_md5 (int i, uint8_t * secret, int secret_len,
   digest_hd_st td;
   int ret;
 
-  ret = _gnutls_hash_init (&td, GNUTLS_MAC_MD5);
+  ret = _gnutls_hash_init (&td, mac_to_entry(GNUTLS_MAC_MD5));
   if (ret < 0)
     {
       gnutls_assert ();
@@ -518,7 +467,7 @@ _gnutls_ssl3_hash_md5 (const void *first, int first_len,
   int block = MD5_DIGEST_OUTPUT;
   int rc;
 
-  rc = _gnutls_hash_init (&td, GNUTLS_MAC_MD5);
+  rc = _gnutls_hash_init (&td, mac_to_entry(GNUTLS_MAC_MD5));
   if (rc < 0)
     {
       gnutls_assert ();
