@@ -182,13 +182,12 @@ typedef enum record_flush_t
 #define RECORD_HEADER_SIZE(session) (IS_DTLS(session) ? DTLS_RECORD_HEADER_SIZE : TLS_RECORD_HEADER_SIZE)
 #define MAX_RECORD_HEADER_SIZE DTLS_RECORD_HEADER_SIZE
 
-#define MAX_USER_SEND_SIZE(session) (IS_DTLS(session)?((size_t)gnutls_dtls_get_data_mtu(session)):(size_t)session->security_parameters.max_record_send_size)
-#define MAX_RECORD_SEND_SIZE(session) (IS_DTLS(session)?((size_t)gnutls_dtls_get_mtu(session)):(size_t)session->security_parameters.max_record_send_size+MAX_RECORD_OVERHEAD)
+#define MAX_RECORD_SEND_SIZE(session) (IS_DTLS(session)?((size_t)gnutls_dtls_get_mtu(session)):(size_t)session->security_parameters.max_record_send_size+MAX_RECORD_OVERHEAD(session))
 #define MAX_RECORD_RECV_SIZE(session) ((size_t)session->security_parameters.max_record_recv_size)
 #define MAX_PAD_SIZE 255
 #define EXTRA_COMP_SIZE 2048
-#define MAX_RECORD_OVERHEAD (MAX_CIPHER_BLOCK_SIZE/*iv*/+MAX_PAD_SIZE+EXTRA_COMP_SIZE+MAX_HASH_SIZE/*MAC*/)
-#define MAX_RECV_SIZE(session) (MAX_RECORD_OVERHEAD+MAX_RECORD_RECV_SIZE(session)+RECORD_HEADER_SIZE(session))
+#define MAX_RECORD_OVERHEAD(session) (MAX_CIPHER_BLOCK_SIZE/*iv*/+MAX_PAD_SIZE+(gnutls_compression_get(session)!=GNUTLS_COMP_NULL)?EXTRA_COMP_SIZE:0+MAX_HASH_SIZE/*MAC*/)
+#define MAX_RECV_SIZE(session) (MAX_RECORD_OVERHEAD(session)+MAX_RECORD_RECV_SIZE(session)+RECORD_HEADER_SIZE(session))
 
 #define TLS_HANDSHAKE_HEADER_SIZE 4
 #define DTLS_HANDSHAKE_HEADER_SIZE (TLS_HANDSHAKE_HEADER_SIZE+8)
@@ -272,6 +271,7 @@ typedef enum extensions_t
   GNUTLS_EXTENSION_SIGNATURE_ALGORITHMS = 13,
   GNUTLS_EXTENSION_SRTP = 14,
   GNUTLS_EXTENSION_HEARTBEAT = 15,
+  GNUTLS_EXTENSION_ALPN = 16,
   GNUTLS_EXTENSION_SESSION_TICKET = 35,
   GNUTLS_EXTENSION_NEW_RECORD_PADDING = 48015,  /* aka: 0xbeaf */
   GNUTLS_EXTENSION_SAFE_RENEGOTIATION = 65281   /* aka: 0xff01 */
@@ -340,7 +340,7 @@ typedef struct mbuffer_st
 
   /* used during fill in, to separate header from data
    * body. */
-  unsigned int user_mark;
+  unsigned int uhead_mark;
 
   /* Filled in by record layer on recv:
    * type, record_sequence
@@ -449,6 +449,46 @@ typedef struct record_state_st record_state_st;
 struct record_parameters_st;
 typedef struct record_parameters_st record_parameters_st;
 
+/* cipher and mac parameters */
+typedef struct cipher_entry_st
+{
+  const char *name;
+  gnutls_cipher_algorithm_t id;
+  uint16_t blocksize;
+  uint16_t keysize;
+  unsigned block:1;
+  uint16_t iv; /* the size of IV */
+  unsigned aead:1; /* Whether it is authenc cipher */
+} cipher_entry_st;
+
+typedef struct mac_entry_st
+{
+  const char *name;
+  const char *oid; /* OID of the hash - if it is a hash */
+  gnutls_mac_algorithm_t id;
+  unsigned output_size;
+  unsigned key_size;
+  unsigned nonce_size;
+  unsigned placeholder; /* if set, then not a real MAC */
+  unsigned secure; /* if set the this algorithm is secure as hash */
+  unsigned block_size; /* internal block size for HMAC */
+} mac_entry_st;
+
+typedef struct
+{
+  const char *name;
+  gnutls_protocol_t id;         /* gnutls internal version number */
+  uint8_t major;                    /* defined by the protocol */
+  uint8_t minor;                    /* defined by the protocol */
+  transport_t transport;	/* Type of transport, stream or datagram */
+  unsigned int supported:1;     /* 0 not supported, > 0 is supported */
+  unsigned int explicit_iv:1;
+  unsigned int extensions:1; /* whether it supports extensions */
+  unsigned int selectable_sighash:1; /* whether signatures can be selected */
+  unsigned int selectable_prf:1; /* whether the PRF is ciphersuite-defined */
+} version_entry_st;
+
+
 /* STATE (cont) */
 
 #include <gnutls_hash_int.h>
@@ -518,7 +558,7 @@ typedef struct
   time_t timestamp;
   
   /* if non-zero the new record padding is used */
-  unsigned int new_record_padding;
+  uint8_t new_record_padding;
 
   /* The send size is the one requested by the programmer.
    * The recv size is the one negotiated with the peer.
@@ -528,19 +568,20 @@ typedef struct
   /* holds the negotiated certificate type */
   gnutls_certificate_type_t cert_type;
   gnutls_ecc_curve_t ecc_curve; /* holds the first supported ECC curve requested by client */
-  gnutls_protocol_t version;    /* moved here */
 
   /* Holds the signature algorithm used in this session - If any */
-  gnutls_sign_algorithm_t sign_algo;
+  gnutls_sign_algorithm_t server_sign_algo;
+  gnutls_sign_algorithm_t client_sign_algo;
 
   /* FIXME: The following are not saved in the session storage
    * for session resumption.
    */
-
+  
   /* Used by extensions that enable supplemental data: Which ones
    * do that? Do they belong in security parameters?
    */
   int do_recv_supplemental, do_send_supplemental;
+  const version_entry_st* pversion;
 } security_parameters_st;
 
 struct record_state_st
@@ -551,8 +592,10 @@ struct record_state_st
   auth_cipher_hd_st cipher_state;
   comp_hd_st compression_state;
   uint64 sequence_number;
+  uint8_t new_record_padding;
 };
 
+
 /* These are used to resolve relative epochs. These values are just
    outside the 16 bit range to prevent off-by-one errors. An absolute
    epoch may be referred to by its numeric id in the range
@@ -566,10 +609,11 @@ struct record_parameters_st
   uint16_t epoch;
   int initialized;
 
-  gnutls_cipher_algorithm_t cipher_algorithm;
-  gnutls_mac_algorithm_t mac_algorithm;
   gnutls_compression_method_t compression_algorithm;
 
+  const cipher_entry_st* cipher;
+  const mac_entry_st* mac;
+
   /* for DTLS */
   uint64_t record_sw[DTLS_RECORD_WINDOW_SIZE];
   unsigned int record_sw_head_idx;
@@ -620,6 +664,7 @@ struct gnutls_priority_st
   unsigned int ssl3_record_version:1;
   unsigned int server_precedence:1;
   unsigned int allow_weak_keys:1;
+  unsigned int allow_wrong_pms:1;
   /* Whether stateless compression will be used */
   unsigned int stateless_compression:1;
   unsigned int additional_verify_flags;
@@ -639,6 +684,7 @@ struct gnutls_priority_st
 
 #define ENABLE_COMPAT(x) \
               (x)->allow_large_records = 1; \
+              (x)->allow_wrong_pms = 1; \
               (x)->allow_weak_keys = 1
 
 /* DH and RSA parameters types.
@@ -656,8 +702,6 @@ typedef struct
 {
   gnutls_dh_params_t dh_params;
   int free_dh_params;
-  gnutls_rsa_params_t rsa_params;
-  int free_rsa_params;
 } internal_params_st;
 
 /* DTLS session state
@@ -705,7 +749,6 @@ typedef struct
   unsigned int packets_dropped;
 } dtls_st;
 
-
 typedef union
 {
   void *ptr;
@@ -780,7 +823,7 @@ typedef struct
                                          * function.
                                          */
 
-  int expire_time;              /* after expire_time seconds this session will expire */
+  unsigned expire_time;              /* after expire_time seconds this session will expire */
   struct mod_auth_st_int *auth_struct;  /* used in handshake packets and KX algorithms */
 
   /* this is the highest version available
@@ -793,9 +836,10 @@ typedef struct
 
   /* if this is non zero a certificate request message
    * will be sent to the client. - only if the ciphersuite
-   * supports it.
+   * supports it. In server side it contains GNUTLS_CERT_REQUIRE
+   * or similar.
    */
-  int send_cert_req;
+  unsigned send_cert_req;
 
   size_t max_handshake_data_buffer_size;
 
@@ -823,15 +867,19 @@ typedef struct
   /* post client hello callback (server side only)
    */
   gnutls_handshake_post_client_hello_func user_hello_func;
+  /* handshake hook function */
+  gnutls_handshake_hook_func h_hook;
+  unsigned int h_type; /* the hooked type */
+  int16_t h_post; /* whether post-generation/receive */
 
   /* holds the selected certificate and key.
    * use _gnutls_selected_certs_deinit() and _gnutls_selected_certs_set()
    * to change them.
    */
   gnutls_pcert_st *selected_cert_list;
-  int selected_cert_list_length;
+  int16_t selected_cert_list_length;
   struct gnutls_privkey_st *selected_key;
-  int selected_need_free:1;
+  unsigned selected_need_free:1;
 
   /* holds the extensions we sent to the peer
    * (in case of a client)
@@ -842,7 +890,7 @@ typedef struct
   /* is 0 if we are to send the whole PGP key, or non zero
    * if the fingerprint is to be sent.
    */
-  int pgp_fingerprint;
+  unsigned pgp_fingerprint:1;
 
   /* This holds the default version that our first
    * record packet will have. */
@@ -850,7 +898,7 @@ typedef struct
 
   void *user_ptr;
 
-  int enable_private;           /* non zero to
+  unsigned enable_private:1;           /* non zero to
                                  * enable cipher suites
                                  * which have 0xFF status.
                                  */
@@ -858,7 +906,7 @@ typedef struct
   /* Holds 0 if the last called function was interrupted while
    * receiving, and non zero otherwise.
    */
-  int direction;
+  unsigned direction:1;
 
   /* This callback will be used (if set) to receive an
    * openpgp key. (if the peer sends a fingerprint)
@@ -868,7 +916,7 @@ typedef struct
   /* If non zero the server will not advertise the CA's he
    * trusts (do not send an RDN sequence).
    */
-  int ignore_rdn_sequence;
+  unsigned ignore_rdn_sequence:1;
 
   /* This is used to set an arbitary version in the RSA
    * PMS secret. Can be used by clients to test whether the
@@ -910,14 +958,14 @@ typedef struct
   {
     uint16_t type;
     extension_priv_data_t priv;
-    int set:1;
+    unsigned set:1;
   } extension_int_data[MAX_EXT_TYPES];
 
   struct
   {
     uint16_t type;
     extension_priv_data_t priv;
-    int set:1;
+    unsigned set:1;
   } resumed_extension_int_data[MAX_EXT_TYPES];
   /* The type of transport protocol; stream or datagram */
   transport_t transport;
@@ -944,13 +992,16 @@ typedef struct
   unsigned int hb_retrans_timeout_ms; /* the default timeout, in milliseconds*/
   unsigned int hb_total_timeout_ms; /* the total timeout, in milliseconds*/
   
-  unsigned int ocsp_check_ok; /* will be zero if the OCSP response TLS extension
+  unsigned int ocsp_check_ok:1; /* will be zero if the OCSP response TLS extension
                                * check failed (OCSP was old/unrelated or so). */
 
   heartbeat_state_t hb_state; /* for ping */
   
   recv_state_t recv_state; /* state of the receive function */
   
+  unsigned int sc_random_set:1;
+  unsigned int no_replay_protection:1; /* DTLS replay protection */
+  
   /* If you add anything here, check _gnutls_handshake_internal_state_clear().
    */
 } internals_st;
@@ -984,28 +1035,52 @@ void _gnutls_free_auth_info (gnutls_session_t session);
 	session->internals.adv_version_major = major; \
 	session->internals.adv_version_minor = minor
 
-void _gnutls_set_adv_version (gnutls_session_t, gnutls_protocol_t);
-gnutls_protocol_t _gnutls_get_adv_version (gnutls_session_t);
-
 int _gnutls_is_secure_mem_null (const void *);
 
-inline static gnutls_protocol_t
-_gnutls_protocol_get_version (gnutls_session_t session)
+inline static const version_entry_st*
+get_version (gnutls_session_t session)
 {
-  return session->security_parameters.version;
+  return session->security_parameters.pversion;
 }
 
-#define gnutls_protocol_get_version _gnutls_protocol_get_version
-
-inline static void
-_gnutls_set_current_version (gnutls_session_t session,
-                             gnutls_protocol_t version)
+inline static unsigned
+get_num_version (gnutls_session_t session)
 {
-  session->security_parameters.version = version;
+  if (likely(session->security_parameters.pversion != NULL))
+    return session->security_parameters.pversion->id;
+  else
+    return GNUTLS_VERSION_UNKNOWN;
 }
 
+#define _gnutls_set_current_version(s, v) { \
+  s->security_parameters.pversion = version_to_entry(v); \
+  }
+
+#define timespec_sub_ms _gnutls_timespec_sub_ms
 unsigned int
 /* returns a-b in ms */
 timespec_sub_ms (struct timespec *a, struct timespec *b);
 
+#include <algorithms.h>
+inline static size_t max_user_send_size(gnutls_session_t session, record_parameters_st *record_params)
+{
+size_t max;
+
+  if (IS_DTLS(session))
+    max = gnutls_dtls_get_data_mtu(session);
+  else
+    {
+      max = session->security_parameters.max_record_send_size;
+      /* DTLS data MTU accounts for those */
+
+      if (record_params->write.new_record_padding != 0)
+        max -= 2;
+
+      if (_gnutls_cipher_is_block (record_params->cipher))
+        max -= _gnutls_cipher_get_block_size(record_params->cipher);
+    }
+
+  return max;
+}
+
 #endif /* GNUTLS_INT_H */
