@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -176,6 +176,7 @@ int gnutls_pkcs11_obj_get_info (gnutls_pkcs11_obj_t crt,
  * gnutls_pkcs11_obj_attr_t:
  * @GNUTLS_PKCS11_OBJ_ATTR_CRT_ALL: Specify all certificates.
  * @GNUTLS_PKCS11_OBJ_ATTR_CRT_TRUSTED: Specify all certificates marked as trusted.
+ * @GNUTLS_PKCS11_OBJ_ATTR_CRT_TRUSTED_CA: Specify all certificates marked as trusted and are CAs.
  * @GNUTLS_PKCS11_OBJ_ATTR_CRT_WITH_PRIVKEY: Specify all certificates with a corresponding private key.
  * @GNUTLS_PKCS11_OBJ_ATTR_PUBKEY: Specify all public keys.
  * @GNUTLS_PKCS11_OBJ_ATTR_PRIVKEY: Specify all private keys.
@@ -190,7 +191,8 @@ typedef enum
     GNUTLS_PKCS11_OBJ_ATTR_CRT_WITH_PRIVKEY,      /* certificates with corresponding private key */
     GNUTLS_PKCS11_OBJ_ATTR_PUBKEY,        /* public keys */
     GNUTLS_PKCS11_OBJ_ATTR_PRIVKEY,       /* private keys */
-    GNUTLS_PKCS11_OBJ_ATTR_ALL    /* everything! */
+    GNUTLS_PKCS11_OBJ_ATTR_ALL,    /* everything! */
+    GNUTLS_PKCS11_OBJ_ATTR_CRT_TRUSTED_CA,       /* CAs */
   } gnutls_pkcs11_obj_attr_t;
 
 /**
@@ -302,6 +304,7 @@ int gnutls_pkcs11_privkey_import_url (gnutls_pkcs11_privkey_t pkey,
 int gnutls_pkcs11_privkey_export_url (gnutls_pkcs11_privkey_t key,
 				      gnutls_pkcs11_url_type_t detailed,
 				      char **url);
+int gnutls_pkcs11_privkey_status (gnutls_pkcs11_privkey_t key);
 
 int gnutls_pkcs11_privkey_generate (const char* url,
 				    gnutls_pk_algorithm_t pk,
@@ -315,6 +318,11 @@ gnutls_pkcs11_privkey_generate2 (const char* url, gnutls_pk_algorithm_t pk,
                                 gnutls_datum_t * pubkey,
                                 unsigned int flags);
 
+int
+gnutls_pkcs11_token_get_random (const char* token_url,
+                                void* data,
+                                size_t len);
+
 #ifdef __cplusplus
 }
 #endif
