@@ -38,7 +38,7 @@
 #define MAX_PROVIDERS 16
 
 /* XXX: try to eliminate this */
-#define MAX_CERT_SIZE 32*1024
+#define MAX_CERT_SIZE 8*1024
 #define MAX_SLOTS 48
 
 struct gnutls_pkcs11_provider_s
@@ -381,7 +381,7 @@ pkcs11_get_info (struct p11_kit_uri *info,
           *output_size = length * 3;
           return GNUTLS_E_SHORT_MEMORY_BUFFER;
         }
-      if (output && length > 0)
+      if (output)
         _gnutls_bin2hex (data, length, output, *output_size, ":");
       *output_size = length * 3;
       return 0;
@@ -992,13 +992,6 @@ _pkcs11_traverse_tokens (find_func_t find_func, void *input,
               continue;
             }
 
-          if (info != NULL)
-            {
-              if (!p11_kit_uri_match_token_info (info, &tinfo.tinfo) ||
-                  !p11_kit_uri_match_module_info (info, &providers[x].info))
-                continue;
-            }
-
           rv = (module)->C_OpenSession (slots[z],
                                         ((flags & SESSION_WRITE)
                                           ? CKF_RW_SESSION : 0) |
@@ -3134,6 +3127,15 @@ pkcs11_set_pin (struct ck_function_list *module,
 	return (module)->C_SetPIN (sess, (uint8_t*)old_pin, old_len, (uint8_t*)new_pin, new_len);
 }
 
+ck_rv_t
+pkcs11_get_random (struct ck_function_list *module,
+                   ck_session_handle_t sess,
+                   void * data,
+                   size_t len)
+{
+	return (module)->C_GenerateRandom (sess, data, len);
+}
+
 const char *
 pkcs11_strerror (ck_rv_t rv)
 {
