@@ -55,9 +55,6 @@ struct node_st {
 struct gnutls_x509_trust_list_st {
   unsigned int size;
   struct node_st *node;
-
-  gnutls_x509_crt_t *blacklisted;
-  unsigned int blacklisted_size;
 };
 
 #define DEFAULT_SIZE 127
@@ -118,11 +115,6 @@ gnutls_x509_trust_list_deinit(gnutls_x509_trust_list_t list,
     if (!list)
         return;
 
-    for (j = 0; j < list->blacklisted_size; j++) {
-		gnutls_x509_crt_deinit(list->blacklisted[j]);
-    }
-    gnutls_free(list->blacklisted);
-
     for (i = 0; i < list->size; i++) {
         if (all)
             for (j = 0; j < list->node[i].trusted_ca_size; j++) {
@@ -192,26 +184,6 @@ gnutls_x509_trust_list_add_cas(gnutls_x509_trust_list_t list,
     return i;
 }
 
-static gnutls_x509_crt_t crt_cpy(gnutls_x509_crt_t src)
-{
-gnutls_x509_crt_t dst;
-int ret;
-
-	ret = gnutls_x509_crt_init(&dst);
-	if (ret < 0) {
-		gnutls_assert();
-		return NULL;
-	}
-	
-	ret = _gnutls_x509_crt_cpy(dst, src);
-	if (ret < 0) {
-		gnutls_assert();
-		return NULL;
-	}
-	
-	return dst;
-}
-
 /**
  * gnutls_x509_trust_list_remove_cas:
  * @list: The structure of the list
@@ -219,13 +191,7 @@ int ret;
  * @clist_size: The length of the CA list
  *
  * This function will remove the given certificate authorities
- * from the trusted list.
- *
- * Note that this function can accept certificates and authorities
- * not yet known. In that case they will be kept in a separate
- * black list that will be used during certificate verification.
- * Unlike gnutls_x509_trust_list_add_cas() there is no deinitialization
- * restriction for  certificate list provided in this function.
+ * from the trusted list. 
  *
  * Returns: The number of removed elements is returned.
  *
@@ -256,23 +222,6 @@ gnutls_x509_trust_list_remove_cas(gnutls_x509_trust_list_t list,
 		r++;
               }
           }
-        
-        /* Add the CA (or plain) certificate to the black list as well.
-	 * This will prevent a subordinate CA from being valid, and 
-	 * ensure that a server certificate will also get rejected.
-	  */
-	list->blacklisted =
-		    gnutls_realloc_fast(list->blacklisted,
-				(list->blacklisted_size + 1) *
-				sizeof(list->blacklisted[0]));
-	if (list->blacklisted == NULL)
-		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-
-	list->blacklisted[list->blacklisted_size] = crt_cpy(clist[i]);
-	if (list->blacklisted[list->blacklisted_size] != NULL)
-		list->blacklisted_size++;
-  
-         
       }
 
     return r;
@@ -563,26 +512,6 @@ int gnutls_x509_trust_list_get_issuer(gnutls_x509_trust_list_t list,
     return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 }
 
-static
-int check_if_in_blacklist(gnutls_x509_crt_t * cert_list, unsigned int cert_list_size,
-	gnutls_x509_crt_t * blacklist, unsigned int blacklist_size)
-{
-unsigned i, j;
-
-	if (blacklist_size == 0)
-		return 0;
-	
-	for (i=0;i<cert_list_size;i++) {
-		for (j=0;j<blacklist_size;j++) {
-			if (_gnutls_check_if_same_cert(cert_list[i], blacklist[j]) != 0) {
-				return 1;
-			}
-		}
-	}
-
-	return 0;
-}
-
 /**
  * gnutls_x509_trust_list_verify_crt:
  * @list: The structure of the list
@@ -596,8 +525,6 @@ unsigned i, j;
  * its status. The @verify parameter will hold an OR'ed sequence of
  * %gnutls_certificate_status_t flags.
  *
- * Limitation: Pathlen constraints or key usage flags are not consulted.
- *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  *
@@ -630,35 +557,12 @@ gnutls_x509_trust_list_verify_crt(gnutls_x509_trust_list_t list,
     	cert_list[cert_list_size - 1]->raw_issuer_dn.size);
     hash %= list->size;
 
-    ret = check_if_in_blacklist(cert_list, cert_list_size,
-		list->blacklisted, list->blacklisted_size);
-    if (ret != 0) {
-	*verify |= GNUTLS_CERT_REVOKED;
-	*verify |= GNUTLS_CERT_INVALID;
-	return 0;
-    }
-
     *verify = _gnutls_x509_verify_certificate(cert_list, cert_list_size,
                                               list->node[hash].trusted_cas,
                                               list->node[hash].
                                               trusted_ca_size, flags,
                                               func);
 
-#define LAST_DN cert_list[cert_list_size-1]->raw_dn
-#define LAST_IDN cert_list[cert_list_size-1]->raw_issuer_dn
-
-    if ((*verify) & GNUTLS_CERT_SIGNER_NOT_FOUND &&
-      (LAST_DN.size != LAST_IDN.size || memcmp(LAST_DN.data, LAST_IDN.data, LAST_IDN.size) != 0)) 
-      {
-        hash = hash_pjw_bare(cert_list[cert_list_size - 1]->raw_dn.data, cert_list[cert_list_size - 1]->raw_dn.size);
-        hash %= list->size;
-        *verify = _gnutls_x509_verify_certificate(cert_list, cert_list_size,
-                                              list->node[hash].trusted_cas,
-                                              list->node[hash].
-                                              trusted_ca_size, flags,
-                                              func);
-      }
-
     if (*verify != 0 || (flags & GNUTLS_VERIFY_DISABLE_CRL_CHECKS))
         return 0;
 
@@ -691,7 +595,6 @@ gnutls_x509_trust_list_verify_crt(gnutls_x509_trust_list_t list,
     }
 
     return 0;
-
 }
 
 /**
@@ -730,14 +633,6 @@ gnutls_x509_trust_list_verify_named_crt(gnutls_x509_trust_list_t list,
     hash = hash_pjw_bare(cert->raw_issuer_dn.data, cert->raw_issuer_dn.size);
     hash %= list->size;
 
-    ret = check_if_in_blacklist(&cert, 1,
-		list->blacklisted, list->blacklisted_size);
-    if (ret != 0) {
-	*verify |= GNUTLS_CERT_REVOKED;
-	*verify |= GNUTLS_CERT_INVALID;
-	return 0;
-    }
-
     *verify = GNUTLS_CERT_INVALID | GNUTLS_CERT_SIGNER_NOT_FOUND;
 
     for (i = 0; i < list->node[hash].named_cert_size; i++) {
