@@ -151,7 +151,7 @@ _gnutls_x509_verify_algorithm (gnutls_digest_algorithm_t * hash,
                                gnutls_pk_algorithm_t pk,
                                gnutls_pk_params_st * issuer_params);
 
-int _gnutls_x509_verify_data (gnutls_digest_algorithm_t algo,
+int _gnutls_x509_verify_data (const mac_entry_st* me,
                               const gnutls_datum_t * data,
                               const gnutls_datum_t * signature,
                               gnutls_x509_crt_t issuer);
